@@ -45,8 +45,8 @@ function (length.out = 1L,
 		gamma <- gamma * n
 		delta <- delta * 1
 		i.S <- 1L
-		i.E <- seq.int(    2L, length.out = m)
-		i.I <- seq.int(m + 2L, length.out = n)
+		i.E <- seq.int(from =     2L, length.out = m)
+		i.I <- seq.int(from = m + 2L, length.out = n)
 		i.R <- p
 	}
 
@@ -96,7 +96,7 @@ function (length.out = 1L,
 			               length.out = p)
 			k.2 <- seq.int(from = (p + 2L) * (p + 2L) + 2L, by = p + 3L,
 			               length.out = p - 1L)
-			D[k.2] <- rep.int(c(sigma, gamma, delta), c(m, n, 1L))
+			D[k.2] <- rep(c(sigma, gamma, delta), c(m, n, 1L))
 
 			ff <-
 			function (x, theta, t)
@@ -143,7 +143,7 @@ function (length.out = 1L,
 			tl.params    = tl.params)
 		m. <- nrow(x.)
 
-		i <- m. - match(seq.int(0L, length.out = length.out),
+		i <- m. - match(seq.int(from = 0L, length.out = length.out),
 		                as.integer(ceiling(x.[m.:1L, 1L]))) + 1L
 		if (anyNA(i)) {
 			## tl.params[["maxtau"]] constrains leaps but not steps => LOCF
@@ -152,7 +152,7 @@ function (length.out = 1L,
 			ik <- i[k]
 			w <- which(ik == 0L)
 			ik[w] <- ik[w - 1L]
-			i <- rep.int(ik, k - c(0L, k[-length(k)]))
+			i <- rep(ik, k - c(0L, k[-length(k)]))
 		}
 		x <- x.[i, -1L, drop = FALSE] # discarding time
 
@@ -206,7 +206,7 @@ function (length.out = 1L,
 			k.0 <- k.1 + p + 2L
 			i.1 <- 2L:(p - 1L)
 			i.0 <- 3L:p
-			a.1 <- rep.int(c(sigma, gamma), c(m, n)); a.11 <- a.1[1L]
+			a.1 <- rep(c(sigma, gamma), c(m, n)); a.11 <- a.1[1L]
 			a.0 <- c(a.1[-1L], delta)
 
 			gg <-
@@ -249,7 +249,7 @@ function (length.out = 1L,
 
 		x. <- deSolve::lsoda(
 			y        = init.,
-			times    = seq.int(0, length.out = length.out),
+			times    = seq.int(from = 0, length.out = length.out),
 			func     = gg,
 			parms    = NULL,
 			jacfunc  = Dg,
@@ -292,8 +292,8 @@ function (length.out = 1L,
 				z <- rbinom(length.out - 1L, z, prob)
 			if (!m.d)
 				## FIXME? 'rmultinom' is more efficient, but not vectorized ...
-				z <- tabulate(rep.int(seq_len(length.out - 1L), z) +
-				              sample(seq.int(0L, length.out = length(delay)),
+				z <- tabulate(rep(seq_len(length.out - 1L), z) +
+				              sample(seq.int(from = 0L, length.out = length(delay)),
 				                     size = sum(z),
 				                     replace = TRUE,
 				                     prob = delay),
@@ -327,11 +327,184 @@ function (length.out = 1L,
 	tsp(x) <- c(0, length.out - 1, 1)
 	dimnames(x) <-
 		list(NULL,
-		     rep.int(c("S", "E", "I", "R", "Z", "B", "Z.obs"),
-		             c(1L, m, n, 1L, 1L, 1L, if (doObs) 1L else 0L)))
+		     rep(c("S", "E", "I", "R", "Z", "B", "Z.obs"),
+		         c(1L, m, n, 1L, 1L, 1L, if (doObs) 1L else 0L)))
 	x
 }
 
+seir.canonical <-
+function (from = 0, to = from + 1, by = 1,
+          R0, ell = (2 * n)/(3 * n + 1),
+          m = 1L, n = 1L,
+          init = c(1 - p, p), p = .Machine[["double.neg.eps"]],
+          weights = rep(c(1, 0), c(1L, m + n - 1L)),
+          root = c("none", "peak"), aggregate = FALSE, ...)
+{
+	tau <- seq.int(from = from, to = to, by = by)
+	stopifnot(requireNamespace("deSolve"),
+	          length(tau) >= 2L, tau[1L] < tau[2L],
+	          is.integer(m), length(m) == 1L, !is.na(m),
+	          m >= 0L && m < 4096L,
+	          is.integer(n), length(n) == 1L, !is.na(n),
+	          n >= 1L && n < 4096L,
+	          is.double(R0), length(R0) == 1L, is.finite(R0),
+	          R0 > 0,
+	          {
+	              if (m == 0L)
+	                  ell <- 0
+	              TRUE
+	          },
+	          is.double(ell), length(ell) == 1L, is.finite(ell),
+	          m == 0L || (ell > 0 && ell < 1),
+	          is.double(init),
+	          length(init) == 2L,
+	          all(is.finite(init)),
+	          init[1L] >= 0,
+	          init[2L] > 0,
+	          sum(init) <= 1,
+	          is.double(weights),
+	          length(weights) == m + n,
+	          all(is.finite(weights)),
+	          min(weights) >= 0,
+	          sum(weights) > 0)
+	root <- match.arg(root)
+	p <- init[2L]
+	init <- c(init[1L], log(weights) - log(sum(weights)) + log(p), p,
+	          use.names = FALSE)
+	if (min(init) == -Inf)
+		init[init == -Inf] <- log(0x1p-64) + log(p)
+
+	i.S <- 1L
+	i.I <- (1L + m + 1L):(1L + m + n)
+	i.1 <- seq.int(from = 2L, length.out = 1L + m + n - 2L)
+	i.2 <- seq.int(from = 3L, length.out = 1L + m + n - 2L)
+
+	h <- (n + 1)/2/n
+	q.1 <- 1 * m/ell
+	q.2 <- h * n/(1 - ell)
+	q.3 <- if (m) q.1 else q.2
+	q.4 <- R0 * (q.5 <- h/(1 - ell))
+	a.1 <-        rep(c(q.1, q.2), c(m, n - 1L))
+	a.2 <- if (m) rep(c(q.1, q.2), c(m - 1L, n)) else a.1
+
+	## D[i, j] = d(rate of change in state i)/d(state j)
+	##
+	## nonzero pattern in m = 4, n = 4 case :
+	##
+	##       S E E E E I I I I Y
+	##     S | . . . . | | | | .
+	##     E | | . . . | | | | .
+	##     E . | | . . . . . . .
+	##     E . . | | . . . . . .
+	##     E . . . | | . . . . .
+	##     I . . . . | | . . . .
+	##     I . . . . . | | . . .
+	##     I . . . . . . | | . .
+	##     I . . . . . . . | | .
+	##     Y | . . . . | | | | .
+	##
+	## nonzero pattern in m = 0, n = 4 case :
+	##
+	##       S I I I I Y
+	##     S | | | | | .
+	##     I | | | | | .
+	##     I . | | . . .
+	##     I . . | | . .
+	##     I . . . | | .
+	##     Y | | | | | .
+	##
+	## where log(.) is suppressed only for pretty printing
+
+	p <- 1L + m + n + 1L
+	D <- matrix(0, p, p)
+	k.1 <- seq.int(from = p + 3L, by = p + 1L, length.out = p - 3L)
+	k.2 <- k.1 + p
+
+	gg <-
+	function (t, x, theta)
+	{
+		x.S <- x[i.S]
+		x.I <- x[i.I]
+		s.1 <- sum(exp(x.I        ))
+		s.2 <- sum(exp(x.I - x[2L]))
+		list(c(-q.4 * s.1 * x.S,
+		       q.4 * s.2 * x.S - q.3,
+		       a.1 * exp(x[i.1] - x[i.2]) - a.2,
+		       q.4 * s.1 * x.S - q.5 * s.1))
+	}
+	Dg <-
+	function (t, x, theta)
+	{
+		x.S <- x[i.S]
+		x.I <- x[i.I]
+		s.1 <- sum(u.1 <- exp(x.I        ))
+		s.2 <- sum(u.2 <- exp(x.I - x[2L]))
+		D[i.S, i.S] <<- -q.4 * s.1
+		D[ 2L, i.S] <<-  q.4 * s.2
+		D[  p, i.S] <<-  q.4 * s.1
+		D[i.S, i.I] <<- -q.4 * u.1 * x.S
+		D[ 2L, i.I] <<-  q.4 * u.2 * x.S
+		D[  p, i.I] <<-  q.4 * u.1 * x.S - q.5 * u.1
+		D[ 2L,  2L] <<- -q.4 * (if (m) s.2 else s.2 - 1) * x.S
+		D[k.2] <<- -(D[k.1] <<- a.1 * exp(x[i.1] - x[i.2]))
+		D
+	}
+	Rg <-
+	switch(root,
+	"peak" =
+	function (t, x, theta)
+		q.4 * x[i.S] - q.5
+	)
+
+	x. <- deSolve::lsoda(
+		y        = init,
+		times    = tau,
+		func     = gg,
+		parms    = NULL,
+		jacfunc  = Dg,
+		jactype  = "fullusr",
+		rootfunc = switch(root, "peak" = Rg),
+		hmax     = by,
+		ynames   = FALSE,
+		initfunc = NULL,
+		initpar  = NULL,
+		...)
+	dimnames(x.) <- NULL
+
+	## FIXME: diff(t[length(t) - 1:0]) != by if terminated
+	if (attr(x., "istate")[1L] < 0L)
+		warning("integration terminated due to unsuccessful solver call")
+
+	if (root == "none") {
+		ans <- x.[, -1L, drop = FALSE]
+		ans[, (1L + 1L):(1L + m + n)] <- exp(ans[, (1L + 1L):(1L + m + n)])
+		if (aggregate) {
+			ans <- seir.aggregate(ans, m, n)
+			if (m > 0L)
+			m <- 1L
+			n <- 1L
+		}
+		oldClass(ans) <- c("seir.canonical", "mts", "ts", "matrix", "array")
+		tsp(ans) <- c(x.[c(1L, nrow(x.)), 1L], 1/by)
+		dimnames(ans) <- list(NULL, rep(c("S", "E", "I", "Y"), c(1L, m, n, 1L)))
+	} else {
+		if (is.null(attr(x., "troot")))
+			return(c(tau = NaN, S = NaN, E = NaN, I = NaN, Y = NaN))
+		r <- x.[nrow(x.), ]
+		tau <- r[1L]
+		S <- r[2L]
+		E <- if (m > 0L) sum(E.full <- exp(r[(2L + 1L    ):(2L + m    )])) else NaN
+		I <-             sum(I.full <- exp(r[(2L + 1L + m):(2L + m + n)]))
+		Y <- r[2L + m + n + 1L]
+		ans <- c(tau = tau, S = S, E = E, I = I, Y = Y)
+		if (m > 0L)
+		attr(ans, "E.full") <- E.full
+		attr(ans, "I.full") <- I.full
+		attr(ans, "curvature") <- -q.4 * q.4 * S * I * I + (q.4 * S - q.5) * ((if (m > 0L) q.1 * E.full[m] else q.4 * S * I) - q.2 * I.full[n])
+	}
+	ans
+} # (q.4 * S - 1) * I
+
 seir.R0 <-
 function (beta, nu = 0, mu = 0, sigma = 1, gamma = 1, delta = 0,
           m = 1L, n = 1L, N = 1)
@@ -366,7 +539,7 @@ function (beta, nu = 0, mu = 0, sigma = 1, gamma = 1, delta = 0,
 		R <- (N - S) / (1 + delta * ((if (m > 0L) 1 / sigma else 0) + 1 / gamma))
 		I <- R * delta / (n * gamma)
 		E <- R * delta / (m * sigma)
-		ans <- rep.int(c(S, E, I, R), c(1L, m, n, 1L))
+		ans <- rep(c(S, E, I, R), c(1L, m, n, 1L))
 	}
 	else {
 		N <- nu / mu
@@ -374,12 +547,12 @@ function (beta, nu = 0, mu = 0, sigma = 1, gamma = 1, delta = 0,
 		b <- 1 + mu / (n * gamma)
 		S <- mu / beta * a^m / (1 - b^-n)
 		R <- (N - S) / (a^m * b^n + delta / mu * (a^m * b^n - 1))
-		I <- cumprod(rep.int(c(R       * (delta + mu) / (n * gamma), b), c(1L, n - 1L)))[n:1L]
+		I <- cumprod(rep(c(R       * (delta + mu) / (n * gamma), b), c(1L, n - 1L)))[n:1L]
 		E <- if (m > 0L)
-		     cumprod(rep.int(c(R * b^n * (delta + mu) / (m * sigma), a), c(1L, m - 1L)))[m:1L]
+		     cumprod(rep(c(R * b^n * (delta + mu) / (m * sigma), a), c(1L, m - 1L)))[m:1L]
 		ans <- c(S, E, I, R)
 	}
-	names(ans) <- rep.int(c("S", "E", "I", "R"), c(1L, m, n, 1L))
+	names(ans) <- rep(c("S", "E", "I", "R"), c(1L, m, n, 1L))
 	ans
 }
 
@@ -388,7 +561,7 @@ function (beta, nu = 0, mu = 0, sigma = 1, gamma = 1, delta = 0,
           m = 1L, n = 1L)
 {
 	p <- m + n + 2L
-	nms <- rep.int(c("S", "E", "I", "R"), c(1L, m, n, 1L))
+	nms <- rep(c("S", "E", "I", "R"), c(1L, m, n, 1L))
 	sigma <- sigma * m
 	gamma <- gamma * n
 	delta <- delta * 1
@@ -397,7 +570,7 @@ function (beta, nu = 0, mu = 0, sigma = 1, gamma = 1, delta = 0,
 	i.I <- seq.int(m + 2L, length.out = n)
 	i.R <- p
 	k <- seq.int(from = p + 3L, by = p + 1L, length.out = p - 2L)
-	a <- rep.int(c(sigma, gamma), c(m, n))
+	a <- rep(c(sigma, gamma), c(m, n))
 	D <- array(0, c(p, p), list(nms, nms))
 	D[i.S, i.R] <- delta
 	D[k    ] <- a
@@ -449,3 +622,37 @@ function (length.out = 1L,
 	call[["m"]] <- 0L
 	eval(call, parent.frame())
 }
+
+summary.seir.canonical <-
+function (object, tol = 1e-6, ...)
+{
+	stopifnot(is.double(tol), length(tol) == 1L, !is.na(tol), tol > 0)
+	ans <- c(NaN, NaN)
+	nms <- colnames(object)
+	p <- rowSums(object[, nms == "E" | nms == "I", drop = FALSE])
+	w <- which(p > 0)
+	if (length(w) == 0L || w[1L] != 1L || (end <- w[length(w)]) < 3L)
+		return(ans)
+	p <- log(p[1L:end])
+	ph <- p[1L:(end - 1L)]
+	pt <- p[2L:(end - 0L)]
+	r <- (pt - ph) * tsp(object)[3L]
+	rh <- r[1L:(end - 2L)]
+	rt <- r[2L:(end - 1L)]
+	d <- (rt - rh)/abs(rh)
+	if (any(ok <- rt > 0 & d >= -tol & d < 0)) {
+		rle.ok <- rle(ok)
+		ptr <- c(0L, cumsum(rle.ok$lengths))
+		j <- which.max(rle.ok$lengths * rle.ok$values)
+		i <- (ptr[j] + 1L):ptr[j + 1L]
+		ans[1L] <- rh[i[which.max(d[i])]]
+	}
+	if (any(ok <- rh < 0 & d >= -tol & d < 0)) {
+		rle.ok <- rle(ok)
+		ptr <- c(0L, cumsum(rle.ok$lengths))
+		j <- which.max(rle.ok$lengths * rle.ok$values)
+		i <- (ptr[j] + 1L):ptr[j + 1L]
+		ans[2L] <- rt[i[which.max(d[i])]]
+	}
+	ans
+}
